<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2020 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Html;

use function array_key_exists;
use function array_unshift;
use function call_user_func;
use function call_user_func_array;
use function explode;
use function get_option;
use function in_array;
use function is_array;
use function is_multisite;

use JchOptimize\Container;
use JchOptimize\Core\Admin\MultiSelectItems;

use function ob_get_clean;
use function ob_start;
use function ucfirst;

class Helper
{
    /**
     * @return false|mixed|string
     */
    public static function _($key, $settingName, $defaultValue, ...$aArgs)
    {
        list($function, $proOnly) = static::extract($key);
        if ($proOnly && !\JCH_PRO) {
            ?>
            <div>
                <em style="padding: 5px; background-color: white; border: 1px #ccc;">
                    <?php
            _e('Only available in Pro Version!', 'jch-optimize');
            ?>
                </em>
            </div>
            <?php
            return null;
        }
        $aSavedSettings = get_option('jch-optimize_settings');
        if (!isset($aSavedSettings[$settingName])) {
            $activeValue = $defaultValue;
        } else {
            $activeValue = $aSavedSettings[$settingName];
        }
        $callable = [__CLASS__, $function];
        // prepend $settingName, $activeValue to arguments
        array_unshift($aArgs, $settingName, $activeValue);

        return call_user_func_array($callable, $aArgs);
    }

    /**
     * @return false|string
     */
    public static function description($title, $description, bool $new = \false)
    {
        ob_start();
        ?>
        <div class="title"><?php
        echo $title;
        ?>
            <?php
        if ($description) {
            ?>
                <div class="description">
                    <div><p><?php
            echo $description;
            ?></p></div>
                </div>
            <?php
        }
        ?>

            <?php
        if ($new) {
            ?>
                <span class="badge badge-danger">New!</span>
            <?php
        }
        ?>
        </div>
        <?php
        return ob_get_clean();
    }

    public static function radio($settingName, $activeValue, string $class = '')
    {
        $disabled = 'pro_capture_cache_enable' == $settingName && is_multisite() ? 'disabled' : '';
        ?>
        <fieldset id="jch-optimize_settings_<?php
        echo $settingName;
        ?>" class="btn-group <?php
        echo $class;
        ?>"
                  role="group" aria-label="Radio toggle button group">
            <input type="radio" id="jch-optimize_settings_<?php
        echo $settingName;
        ?>0"
                   name="<?php
        echo 'jch-optimize_settings['.$settingName.']';
        ?>" class="btn-check" value="0"
                    <?php
        echo '0' == $activeValue ? 'checked' : '';
        ?> <?php
        echo $disabled;
        ?> >
            <label for="jch-optimize_settings_<?php
        echo $settingName;
        ?>0"
                   class="btn btn-outline-secondary"><?php
        _e('No', 'jch-optimize');
        ?></label>
            <input type="radio" id="jch-optimize_settings_<?php
        echo $settingName;
        ?>1"
                   name="<?php
        echo 'jch-optimize_settings['.$settingName.']';
        ?>" class="btn-check" value="1"
                    <?php
        echo '1' == $activeValue ? 'checked' : '';
        ?> <?php
        echo $disabled;
        ?> >
            <label for="jch-optimize_settings_<?php
        echo $settingName;
        ?>1"
                   class="btn btn-outline-secondary"><?php
        _e('Yes', 'jch-optimize');
        ?></label>
        </fieldset>
        <?php
        if ('pro_smart_combine' == $settingName) {
            $imgUrl = JCH_PLUGIN_URL.'media/core/images/exclude-loader.gif';
            ?>
            <img id="img-<?php
            echo $settingName;
            ?>" src="<?php
            echo $imgUrl;
            ?>" style="display: none;">
            <button id="btn-<?php
            echo $settingName;
            ?>" type="button" class="btn btn-outline-secondary"
                    style="display: none;">Reprocess Smart Combine
            </button>
            <?php
        }
    }

    public static function select($settingName, $activeValue, $aOptions, string $class = '', array $conditions = [])
    {
        ?>
        <select id="jch-optimize_settings_<?php
        echo $settingName;
        ?>"
                name="<?php
        echo 'jch-optimize_settings['.$settingName.']';
        ?>"
                class="chzn-custom-value <?php
        echo $class;
        ?>">
            <?php
        echo self::option($aOptions, $activeValue, $conditions);
        ?>
        </select>
        <?php
    }

    /**
     * @return false|string
     */
    public static function option($options, $activeValue, array $conditions = [])
    {
        ob_start();
        foreach ($options as $key => $value) {
            $selected = $activeValue == $key ? ' selected' : '';
            $disabled = '';
            if (!empty($conditions && array_key_exists($key, $conditions))) {
                if (!call_user_func($conditions[$key])) {
                    $disabled = ' disabled';
                }
            }
            ?>
            <option value="<?php
            esc_attr_e($key);
            ?>"<?php
            echo $selected;
            echo $disabled;
            ?>><?php
            echo $value;
            ?></option>
            <?php
        }

        return ob_get_clean();
    }

    public static function multiselect($settingName, $aActiveValues, $type, $group, string $class = '')
    {
        $container = Container::getInstance();
        $multiSelect = $container->buildObject(MultiSelectItems::class);
        ?>
        <select id="jch-optimize_settings_<?php
        echo $settingName;
        ?>"
                name="<?php
        echo 'jch-optimize_settings['.$settingName.']';
        ?>[]"
                class="jch-multiselect chzn-custom-value <?php
        echo $class;
        ?>" multiple="multiple" size="5"
                data-jch_type="<?php
        echo $type;
        ?>" data-jch_group="<?php
        echo $group;
        ?>"
                data-jch_param="<?php
        echo $settingName;
        ?>">

            <?php
        foreach ($aActiveValues as $value) {
            $option = $multiSelect->{'prepare'.ucfirst($group).'Values'}($value);
            ?>
                <option value="<?php
            esc_attr_e($value);
            ?>" selected><?php
            echo $option;
            ?></option>
                <?php
        }
        ?>
        </select>
        <img id="img-<?php
        echo $settingName;
        ?>" class="jch-multiselect-loading-image"
             src="<?php
        echo JCH_PLUGIN_URL.'media/core/images/exclude-loader.gif';
        ?>"/>
        <button id="btn-<?php
        echo $settingName;
        ?>" style="display: none;"
                class="btn btn-secondary btn-sm jch-multiselect-add-button" type="button"
                onclick="jchMultiselect.addJchOption('jch-optimize_settings_<?php
        echo $settingName;
        ?>')"><?php
        _e('Add item', 'jch-optimize');
        ?></button>
        <?php
    }

    /**
     * @param string $class
     */
    public static function text($settingName, $activeValue, string $size = '30', $class = '')
    {
        ?>
        <input type="text" id="jch-optimize_settings_<?php
        echo $settingName;
        ?>"
               name="<?php
        echo 'jch-optimize_settings['.$settingName.']';
        ?>"
               value="<?php
        esc_attr_e($activeValue);
        ?>" size="<?php
        echo $size;
        ?>" class="<?php
        echo $class;
        ?>">
        <?php
    }

    /**
     * @param string $class
     */
    public static function checkbox($settingName, $activeValue, $class = '')
    {
        $checked = '1' == $activeValue ? 'checked="checked"' : '';
        ?>
        <input type="checkbox" id="jch-optimize_settings_<?php
        echo $settingName;
        ?>" class="<?php
        echo $class;
        ?>"
               name="<?php
        echo 'jch-optimize_settings['.$settingName.']';
        ?>" data-toggle="toggle"
               data-onstyle="success" data-offstyle="danger" data-on="<?php
        _e('Yes', 'jch-optimize');
        ?>"
               data-off="<?php
        _e('No', 'jch-optimize');
        ?>" value="1" <?php
        echo $checked;
        ?>>
        <?php
    }

    /**
     * @param string $class
     */
    public static function checkboxes($settingName, $activeValues, $options, $class = '')
    {
        $i = '0';
        ?>
        <fieldset id="jch-optimize_settings_<?php
        echo $settingName;
        ?>" class="<?php
        echo $class;
        ?>">
            <ul>
                <?php
        foreach ($options as $key => $value) {
            $checked = is_array($activeValues) && in_array($key, $activeValues) ? 'checked' : '';
            ?>
                    <li>
                        <input type="checkbox" id="jch-optimize_settings_<?php
            echo $settingName.$i;
            ?>"
                               name="<?php
            echo 'jch-optimize_settings['.$settingName.']';
            ?>[]"
                               value="<?php
            esc_attr_e($key);
            ?>" <?php
            echo $checked;
            ?>>
                        <label for="jch-optimize_settings_<?php
            echo $settingName.$i;
            ?>">
                            <?php
            echo $value;
            ?>
                        </label>

                    </li>
                    <?php
            ++$i;
        }
        ?>
            </ul>
        </fieldset>
        <?php
    }

    protected static function extract($key): array
    {
        $parts = explode('.', $key);
        $function = $parts[0];
        $proOnly = isset($parts[1]) && 'pro' === $parts[1];

        return [$function, $proOnly];
    }
}
