<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2020 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Plugin;

use _JchOptimizeVendor\Joomla\Input\Input;
use _JchOptimizeVendor\Joomla\Registry\Registry;

use function JchOptimize\base64_encode_url;

use JchOptimize\Container;
use JchOptimize\ControllerResolver;
use JchOptimize\Core\Admin\Ajax\Ajax;
use JchOptimize\Html\Helper;
use JchOptimize\Html\TabSettings;
use JchOptimize\Platform\Paths;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\NotFoundExceptionInterface;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;

class Admin implements LoggerAwareInterface
{
    use LoggerAwareTrait;

    /**
     * @var Registry
     */
    private $params;

    /**
     * @var ControllerResolver
     */
    private $controllerResolver;

    public function __construct(Registry $params, ControllerResolver $controllerResolver)
    {
        $this->params = $params;
        $this->controllerResolver = $controllerResolver;
    }

    public static function publishAdminNotices()
    {
        try {
            if ($messages = \get_transient('jch-optimize_notices')) {
                foreach ($messages as $message) {
                    echo <<<HTML
<div class="notice notice-{$message['type']} is-dismissible"><p>{$message['message']}</p></div>
HTML;
                }
                \delete_transient('jch-optimize_notices');
            }
        } catch (\Exception $e) {
        }
    }

    public function addAdminMenu()
    {
        $menuTitle = \JCH_PRO ? 'JCH Optimize Pro' : 'JCH Optimize';
        $hook_suffix = \add_options_page(\__('JCH Optimize Settings', 'jch-optimize'), $menuTitle, 'manage_options', 'jch_optimize', [$this, 'loadAdminPage']);
        \add_action('admin_enqueue_scripts', [$this, 'loadResourceFiles']);
        \add_action('admin_head-'.$hook_suffix, [$this, 'addScriptsToHead']);
        \add_action('load-'.$hook_suffix, [$this, 'initializeSettings']);
        \add_action('admin_bar_menu', [$this, 'addMenuToAdminBar'], 101);
        \add_action('admin_init', [$this, 'checkMessages']);
    }

    public function loadAdminPage()
    {
        try {
            $this->controllerResolver->resolve();
        } catch (NotFoundExceptionInterface|ContainerExceptionInterface $e) {
        } catch (\Exception $e) {
            $class = \get_class($e);
            echo <<<HTML
<h1>Application Error</h1>
<p>Please submit the following error message and trace in a support request:</p>
<div class="alert alert-danger">  {$class} &mdash;  {$e->getMessage()}  </div>
<pre class="well"> {$e->getTraceAsString()} </pre>
HTML;
        }
    }

    public function registerOptions()
    {
        // Buffer output to allow for redirection
        \ob_start();
        \register_setting('jchOptimizeOptionsPage', 'jch-optimize_settings', ['type' => 'array']);
    }

    public function loadResourceFiles($hook)
    {
        if ('settings_page_jch_optimize' != $hook) {
            return;
        }
        \wp_enqueue_style('jch-bootstrap-css');
        \wp_enqueue_style('jch-verticaltabs-css');
        \wp_enqueue_style('jch-admin-css');
        \wp_enqueue_style('jch-fonts-css');
        \wp_enqueue_style('jch-chosen-css');
        \wp_enqueue_style('jch-wordpress-css');
        \wp_enqueue_style('jch-filetree-css');
        \wp_enqueue_script('jch-platformwordpress-js');
        \wp_enqueue_script('jch-bootstrap-js');
        \wp_enqueue_script('jch-adminutility-js');
        \wp_enqueue_script('jch-multiselect-js');
        \wp_enqueue_script('jch-chosen-js');
        \wp_enqueue_script('jch-collapsible-js');
        \wp_enqueue_script('jch-filetree-js');
        if (\JCH_PRO) {
            \wp_enqueue_style('jch-progressbar-css');
            \wp_enqueue_script('jquery-ui-progressbar');
            \wp_enqueue_script('jch-optimizeimage-js');
            \wp_enqueue_script('jch-smartcombine-js');
        }
    }

    public function addScriptsToHead()
    {
        echo <<<HTML
\t\t<style>
                    .chosen-container-multi .chosen-choices li.search-field input[type=text] {
                        height: 25px;
                    }

                    .chosen-container {
                        margin-right: 4px;
                    }

\t\t</style>
\t\t<script type="text/javascript">
\t\t(function(\$){
                    function submitJchSettings() {
                        \$("form.jch-settings-form").submit();
                    }
                })(jQuery);
                </script>
                    
HTML;
        if (\JCH_PRO) {
            $optimizeImageUrl = \add_query_arg(['page' => 'jch_optimize', 'view' => 'optimizeimage'], \admin_url('options-general.php'));
            $paramsArray = $this->params->toArray();
            $aApiParams = ['pro_downloadid' => '', 'hidden_api_secret' => '11e603aa', 'ignore_optimized' => '1', 'recursive' => '1', 'pro_api_resize_mode' => '1', 'pro_next_gen_images' => '1', 'lossy' => '1', 'save_metadata' => '0'];
            $jch_message = \__('Please open a directory to optimize images.', 'jch-optimize');
            $jch_noproid = \__('Please enter your Download ID on the Configurations tab.', 'jch-optimize');
            $jch_params = \json_encode(\array_intersect_key($paramsArray, $aApiParams));
            echo <<<HTML

\t\t<script>
                    var jch_message = '{$jch_message}';
                    var jch_noproid = '{$jch_noproid}';
                    var jch_params = JSON.parse('{$jch_params}');
\t\t</script>
HTML;
        }
    }

    public function initializeSettings()
    {
        // Css files
        \wp_register_style('jch-bootstrap-css', JCH_PLUGIN_URL.'media/bootstrap/css/bootstrap.min.css', [], \JCH_VERSION);
        \wp_register_style('jch-admin-css', JCH_PLUGIN_URL.'media/core/css/admin.css', [], \JCH_VERSION);
        \wp_register_style('jch-fonts-css', 'https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.0.0/css/all.min.css');
        \wp_register_style('jch-chosen-css', JCH_PLUGIN_URL.'media/chosen-js/chosen.css', [], \JCH_VERSION);
        \wp_register_style('jch-wordpress-css', JCH_PLUGIN_URL.'media/css/wordpress.css', [], \JCH_VERSION);
        \wp_register_style('jch-filetree-css', JCH_PLUGIN_URL.'media/filetree/jquery.filetree.css', [], \JCH_VERSION);
        // JavaScript files
        \wp_register_script('jch-bootstrap-js', JCH_PLUGIN_URL.'media/bootstrap/js/bootstrap.bundle.min.js', ['jquery'], \JCH_VERSION, \true);
        \wp_register_script('jch-platformwordpress-js', JCH_PLUGIN_URL.'media/js/platform-wordpress.js', ['jquery'], \JCH_VERSION, \true);
        \wp_register_script('jch-adminutility-js', JCH_PLUGIN_URL.'media/core/js/admin-utility.js', ['jquery'], \JCH_VERSION, \true);
        \wp_register_script('jch-multiselect-js', JCH_PLUGIN_URL.'media/core/js/multiselect.js', ['jquery', 'jch-adminutility-js', 'jch-platformwordpress-js'], \JCH_VERSION, \true);
        \wp_register_script('jch-chosen-js', JCH_PLUGIN_URL.'media/chosen-js/chosen.jquery.js', ['jquery'], \JCH_VERSION, \true);
        \wp_register_script('jch-filetree-js', JCH_PLUGIN_URL.'media/filetree/jquery.filetree.js', ['jquery'], \JCH_VERSION, \true);
        $loader_image = Paths::mediaUrl().'/core/images/loader.gif';
        $imageLoaderJs = <<<JS
const jch_loader_image_url = "{$loader_image}"
JS;
        \wp_add_inline_script('jch-platformwordpress-js', $imageLoaderJs, 'before');
        $chosenJs = <<<JS
\t\t(function(\$){
                   \$(document).ready(function () {
                        \$(".chzn-custom-value").chosen({
                             disable_search_threshold: 10,
                             width: "300px",
                             placeholder_text_multiple: "Select some options or add items to select"
                         });
                   });
                })(jQuery);
JS;
        \wp_add_inline_script('jch-chosen-js', $chosenJs);
        $popoverJs = <<<JS
\t\twindow.onload = function(){
\t\t\tvar popoverTriggerList = [].slice.call(document.querySelectorAll('.hasPopover'));
            \t\tvar popoverList = popoverTriggerList.map(function(popoverTriggerEl){
\t\t     \t\treturn new bootstrap.Popover(popoverTriggerEl, {
\t\t\t\t\thtml: true,
\t\t\t\t\tcontainer: 'body',
\t\t\t\t\tplacement: 'right',
\t\t\t\t\ttrigger: 'hover focus'
\t\t\t\t});
\t\t\t});
            \t}
JS;
        \wp_add_inline_script('jch-bootstrap-js', $popoverJs);
        if (\JCH_PRO) {
            \wp_register_style('jch-progressbar-css', JCH_PLUGIN_URL.'media/jquery-ui/jquery-ui.css', [], \JCH_VERSION);
            \wp_register_script('jch-optimizeimage-js', JCH_PLUGIN_URL.'media/core/js/ioptimize-api.js', ['jquery', 'jch-adminutility-js', 'jch-platformwordpress-js', 'jch-bootstrap-js'], \JCH_VERSION, \true);
            \wp_register_script('jch-progressbar-js', JCH_PLUGIN_URL.'media/jquery-ui/jquery-ui.js', ['jquery'], \JCH_VERSION, \true);
            \wp_register_script('jch-smartcombine-js', JCH_PLUGIN_URL.'media/core/js/smart-combine.js', ['jquery', 'jch-adminutility-js', 'jch-platformwordpress-js'], \JCH_VERSION, \true);
        }
        $aSettingsArray = TabSettings::getSettingsArray();
        foreach ($aSettingsArray as $section => $aSettings) {
            \add_settings_section('jch-optimize_'.$section.'_section', '', ['\\JchOptimize\\Html\\Renderer\\Section', $section], 'jchOptimizeOptionsPage');

            /**
             * $aArgs = [
             *        0 => title,
             *        1 => description,
             *        2 => new
             * ].
             */
            foreach ($aSettings as $setting => $aArgs) {
                list($title, $description, $new) = \array_pad($aArgs, 3, 0);
                $id = 'jch-optimize_'.$setting;
                $title = Helper::description($title, $description, $new);
                $args = [];
                $aClasses = $this->getSettingsClassMap();
                if (isset($aClasses[$setting])) {
                    $args['class'] = $aClasses[$setting];
                }
                \add_settings_field($id, $title, ['\\JchOptimize\\Html\\Renderer\\Setting', $setting], 'jchOptimizeOptionsPage', 'jch-optimize_'.$section.'_section', $args);
            }
        }
    }

    public function addMenuToAdminBar($admin_bar)
    {
        if (!\current_user_can('manage_options')) {
            return;
        }
        $nodes = $admin_bar->get_nodes();
        if (!isset($nodes['jch-optimize-parent'])) {
            $aArgs = ['id' => 'jch-optimize-parent', 'title' => 'JCH Optimize'];
            $admin_bar->add_node($aArgs);
        }
        $aArgs = ['parent' => 'jch-optimize-parent', 'id' => 'jch-optimize-settings', 'title' => \__('Settings', 'jch-optimize'), 'href' => \add_query_arg(['page' => 'jch_optimize'], \admin_url('options-general.php'))];
        $admin_bar->add_node($aArgs);
        $aArgs = ['parent' => 'jch-optimize-parent', 'id' => 'jch-optimize-cache', 'title' => \__('Clean Cache', 'jch-optimize'), 'href' => \add_query_arg(['page' => 'jch_optimize', 'task' => 'cleancache', 'return' => base64_encode_url($this->getCurrentAdminUri())], \admin_url('options-general.php'))];
        $admin_bar->add_node($aArgs);
    }

    public function getCurrentAdminUri()
    {
        $uri = isset($_SERVER['REQUEST_URI']) ? \esc_url_raw(wp_unslash($_SERVER['REQUEST_URI'])) : '';
        $uri = \preg_replace('|^.*/wp-admin/|i', '', $uri);
        if (!$uri) {
            return '';
        }

        return $uri;
    }

    public function checkMessages()
    {
        if (\get_transient('jch-optimize_notices')) {
            \add_action('admin_notices', [__CLASS__, 'publishAdminNotices']);
        }
    }

    public function loadActionLinks($links, $file)
    {
        static $this_plugin;
        if (!$this_plugin) {
            $this_plugin = \plugin_basename(JCH_PLUGIN_FILE);
        }
        if ($file == $this_plugin) {
            $settings_link = '<a href="'.\admin_url('options-general.php?page=jch_optimize').'">'.\__('Settings').'</a>';
            \array_unshift($links, $settings_link);
        }

        return $links;
    }

    public function doAjaxFileTree()
    {
        echo Ajax::getInstance('FileTree')->run();

        exit;
    }

    public function doAjaxMultiSelect()
    {
        echo Ajax::getInstance('MultiSelect')->run();

        exit;
    }

    public function doAjaxOptimizeImages()
    {
        $container = Container::getInstance();
        $oInput = $container->get(Input::class);
        $oInput->def('task', 'optimizeimage');
        $container->get(ControllerResolver::class)->resolve();
    }

    public function doAjaxConfigureSettings()
    {
        $container = Container::getInstance();
        $container->get(ControllerResolver::class)->resolve();
    }

    public function doAjaxSmartCombine()
    {
        echo Ajax::getInstance('SmartCombine')->run();

        exit;
    }

    /**
     * Map of classes that should be put on the '<tr>' element containing the associated setting.
     *
     * @return string[]
     */
    private function getSettingsClassMap()
    {
        return ['pro_http2_file_types' => 'jch-wp-checkboxes-grid-wrapper columns-4', 'staticfiles' => 'jch-wp-checkboxes-grid-wrapper columns-5', 'pro_staticfiles_2' => 'jch-wp-checkboxes-grid-wrapper columns-5', 'pro_staticfiles_3' => 'jch-wp-checkboxes-grid-wrapper columns-5', 'pro_html_sections' => 'jch-wp-checkboxes-grid-wrapper columns-5 width-400', 'memcached_server_host' => 'jch-memcached-wrapper d-none', 'memcached_server_port' => 'jch-memcached-wrapper d-none', 'redis_server_host' => 'jch-redis-wrapper d-none', 'redis_server_port' => 'jch-redis-wrapper d-none', 'redis_server_password' => 'jch-redis-wrapper d-none', 'redis_server_database' => 'jch-redis-wrapper d-none'];
    }
}
