<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2020 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Plugin;

use _JchOptimizeVendor\Joomla\DI\ContainerAwareInterface;
use _JchOptimizeVendor\Joomla\DI\ContainerAwareTrait;
use _JchOptimizeVendor\Joomla\Input\Input;
use _JchOptimizeVendor\Joomla\Registry\Registry;

use function add_filter;

use JchOptimize\Container;
use JchOptimize\Core\Helper;
use JchOptimize\Core\Optimize;
use JchOptimize\Core\PageCache\PageCache;
use JchOptimize\Core\SystemUri;
use JchOptimize\Platform\Utility;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerAwareTrait;
use Psr\Log\LogLevel;

use function register_uninstall_hook;

class Loader implements LoggerAwareInterface, ContainerAwareInterface
{
    use LoggerAwareTrait;
    use ContainerAwareTrait;

    /**
     * @var null|Registry
     */
    private $params;

    /**
     * @var Admin
     */
    private $admin;

    /**
     * @var PageCache
     */
    private $pageCache;

    /**
     * @var Installer
     */
    private $installer;

    /**
     * @var Updater
     */
    private $updater;

    /**
     * @var Optimize
     */
    private $optimize;

    public function __construct(Registry $params, Admin $admin, Installer $installer, ?Updater $updater, Optimize $optimize, PageCache $pageCache)
    {
        $this->params = $params;
        $this->admin = $admin;
        $this->installer = $installer;
        $this->updater = $updater;
        $this->optimize = $optimize;
        $this->pageCache = $pageCache;
    }

    /**
     * Function register_uninstall_hook can only be used with a static method.
     */
    public static function runUninstallRoutines()
    {
        /** @var Installer $installer */
        $installer = Container::getInstance()->get(\JchOptimize\Plugin\Installer::class);
        $installer->deactivate();
    }

    public function preboot_init()
    {
        if (\false !== ($settings = \get_option('jch_options'))) {
            \update_option('jch-optimize_settings', $settings);
            \delete_option('jch_options');
        }
    }

    public function init()
    {
        $this->runActivationRoutines();
        if (\is_admin()) {
            \add_action('admin_menu', [$this->admin, 'addAdminMenu']);
            \add_action('admin_init', [$this->admin, 'registerOptions']);
            \add_filter('plugin_action_links', [$this->admin, 'loadActionLinks'], 10, 2);
        } else {
            $url_exclude = $this->params->get('menuexcludedurl', []);

            /** @var Input $input */
            $input = $this->container->get(Input::class);
            $jch_backend = $input->get('jchbackend');
            if (\defined('WP_USE_THEMES') && \WP_USE_THEMES && 1 != $jch_backend && \version_compare(\PHP_VERSION, '7.3.0', '>=') && !\defined('DOING_AJAX') && !\defined('DOING_CRON') && !\defined('APP_REQUEST') && !\defined('XMLRPC_REQUEST') && (!\defined('SHORTINIT') || \defined('SHORTINIT') && !\SHORTINIT) && !Helper::findExcludes($url_exclude, SystemUri::toString())) {
                // Disable NextGen Resource Manager; incompatible with plugin
                // add_filter( 'run_ngg_resource_manager', '__return_false' );
                \add_action('init', [$this, 'initializeCache'], 0);
                \ob_start([$this, 'runOptimize']);
                if (JCH_DEBUG && !$this->params->get('disable_logged_in_users', '1')) {
                    \add_action('admin_bar_menu', [$this, 'addMenuToAdminBar'], 101);
                }
            }
        }
        \add_action('plugins_loaded', [$this, 'loadPluginTextDomain']);
        // Function register_uninstall_hook can only be used with a static class method
        \register_uninstall_hook(JCH_PLUGIN_FILE, [__CLASS__, 'runUninstallRoutines']);
        if ($this->params->get('order_plugin', '1')) {
            \add_action('activated_plugin', [$this, 'orderPlugin']);
            \add_action('deactivated_plugin', [$this, 'orderPlugin']);
        }
        if (\JCH_PRO) {
            $this->loadProUpdater();
            if ($this->params->get('pro_cache_platform', '0')) {
                \add_filter('jch_optimize_get_page_cache_id', [$this, 'getPageCacheHash'], 10, 2);
            }
        }
        // Ajax functions
        \add_action('wp_ajax_filetree', [$this->admin, 'doAjaxFileTree']);
        \add_action('wp_ajax_multiselect', [$this->admin, 'doAjaxMultiSelect']);
        \add_action('wp_ajax_optimizeimages', [$this->admin, 'doAjaxOptimizeImages']);
        \add_action('wp_ajax_smartcombine', [$this->admin, 'doAjaxSmartCombine']);
        \add_action('wp_ajax_configuresettings', [$this->admin, 'doAjaxConfigureSettings']);
    }

    public function runActivationRoutines()
    {
        // Handles activation routines
        \register_activation_hook(JCH_PLUGIN_FILE, [$this->installer, 'activate']);
        $mu_folder = \ABSPATH.'wp-content/mu-plugins';
        if (\defined('WPMU_PLUGIN_DIR') && \WPMU_PLUGIN_DIR) {
            $mu_folder = \WPMU_PLUGIN_DIR;
        }
        if (!\file_exists(\JCH_PLUGIN_DIR.'dir.php') || \JCH_PRO && \is_admin() && !\file_exists($mu_folder.'/jch-optimize-mode-switcher.php')) {
            $this->installer->activate();
        }
    }

    public function initializeCache()
    {
        $this->pageCache->initialize();
        $this->pageCache->setCaching();
    }

    public function runOptimize($html)
    {
        if (!Helper::validateHtml($html)) {
            return $html;
        }
        // need to check this here, it could be set dynamically
        global $jch_no_optimize;
        $disable_logged_in = $this->params->get('disable_logged_in_users', '1');
        // Need to call Utility::isGuest after init has been called
        if ($jch_no_optimize || $disable_logged_in && !Utility::isGuest()) {
            return $html;
        }

        try {
            $this->optimize->setHtml($html);
            $optimizedHtml = $this->optimize->process();
            // required for compatibility with Hide My WP Ghost https://wordpress.org/support/topic/compatibility-with-hide-my-wp-ghost/
            $optimizedHtml = \apply_filters('jch_optimize_save_content', $optimizedHtml);
            $this->pageCache->store($optimizedHtml);
        } catch (\Exception $e) {
            $this->logger->log(LogLevel::ERROR, $e->getMessage());
            $optimizedHtml = $html;
        }

        return $optimizedHtml;
    }

    public function loadPluginTextDomain()
    {
        \load_plugin_textdomain('jch-optimize', \false, \dirname(\plugin_basename(JCH_PLUGIN_FILE)).'/languages');
    }

    public function orderPlugin(): bool
    {
        $active_plugins = (array) \get_option('active_plugins', []);
        $order = ['jch-optimize/jch-optimize.php'];
        // Get the plugins in $order that are currently activated
        $order_short_list = \array_intersect($order, $active_plugins);
        // Remove plugins in $order_short_list from list of activated plugins
        $active_plugins_slist = \array_diff($active_plugins, $order_short_list);
        // Merge $order with $active_plugins_list
        $ordered_active_plugins = \array_merge($order_short_list, $active_plugins_slist);
        \update_option('active_plugins', $ordered_active_plugins);

        return \true;
    }

    public function getPageCacheHash($parts)
    {
        if (\wp_is_mobile()) {
            $parts[] = '_MOBILE_';
        }

        return $parts;
    }

    public function addMenuToAdminBar($admin_bar)
    {
        if (!\current_user_can('manage_options')) {
            return;
        }
        $nodes = $admin_bar->get_nodes();
        if (!isset($nodes['jch-optimize-parent'])) {
            $aArgs = ['id' => 'jch-optimize-parent', 'title' => \__('JCH Optimize', 'jch-optimize')];
            $admin_bar->add_node($aArgs);
        }
        $aArgs = ['parent' => 'jch-optimize-parent', 'id' => 'jch-optimize-profiler', 'title' => \__('Profiler', 'jch-optimize')];
        $admin_bar->add_node($aArgs);
    }

    protected function loadProUpdater()
    {
        $this->updater->load();
    }
}
