<?php

/**
 * JCH Optimize - Performs several front-end optimizations for fast downloads.
 *
 * @author    Samuel Marshall <samuel@jch-optimize.net>
 * @copyright Copyright (c) 2021 Samuel Marshall / JCH Optimize
 * @license   GNU/GPLv3, or later. See LICENSE file
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

namespace JchOptimize\Service;

use _JchOptimizeVendor\Illuminate\View\Factory;
use _JchOptimizeVendor\Joomla\DI\Container;
use _JchOptimizeVendor\Joomla\DI\ServiceProviderInterface;
use _JchOptimizeVendor\Joomla\Input\Input;
use _JchOptimizeVendor\Joomla\Renderer\BladeRenderer;
use _JchOptimizeVendor\Joomla\Renderer\RendererInterface;
use _JchOptimizeVendor\Joomla\View\HtmlView;
use _JchOptimizeVendor\Laminas\Cache\Storage\StorageInterface;
use _JchOptimizeVendor\Laminas\Cache\Storage\TaggableInterface;
use JchOptimize\Controller\ApplyAutoSetting;
use JchOptimize\Controller\BrowserCaching;
use JchOptimize\Controller\CleanCache;
use JchOptimize\Controller\Configurations;
use JchOptimize\Controller\DeleteBackups;
use JchOptimize\Controller\Help;
use JchOptimize\Controller\KeyCache;
use JchOptimize\Controller\Main;
use JchOptimize\Controller\OptimizeImage;
use JchOptimize\Controller\OptimizeImages;
use JchOptimize\Controller\OrderPlugins;
use JchOptimize\Controller\PageCache;
use JchOptimize\Controller\RestoreImages;
use JchOptimize\Controller\ToggleSetting;
use JchOptimize\ControllerResolver;
use JchOptimize\Core\Admin\Icons;
use JchOptimize\Core\Interfaces\MvcLoggerInterface;
use JchOptimize\Core\PageCache\PageCache as CorePageCache;
use JchOptimize\Log\WordpressNoticeLogger;
use JchOptimize\Model\Cache;
use JchOptimize\Model\Configure;
use JchOptimize\Model\PageCache as PageCacheModel;
use JchOptimize\Platform\Paths;
use JchOptimize\Plugin\Loader;
use JchOptimize\View\ConfigurationsHtml;
use JchOptimize\View\MainHtml;
use JchOptimize\View\PageCacheHtml;

class MvcProvider implements ServiceProviderInterface
{
    public function register(Container $container)
    {
        // MVC dependencies
        $container->share(Input::class, [$this, 'getInputService'], \true);
        $container->alias(MvcLoggerInterface::class, WordpressNoticeLogger::class)->share(WordpressNoticeLogger::class, [$this, 'getWordpressNoticeLoggerService'], \true);
        $container->share(RendererInterface::class, [$this, 'getTemplateRendererService'], \true);
        $container->share(ControllerResolver::class, [$this, 'getControllerResolverService'], \true);
        // controllers
        $container->alias(ApplyAutoSetting::class, 'applyautosetting')->share('applyautosetting', [$this, 'getControllerApplyAutoSettingService'], \true);
        $container->alias(BrowserCaching::class, 'browsercaching')->share('browsercaching', [$this, 'getControllerBrowserCachingService'], \true);
        $container->alias(CleanCache::class, 'cleancache')->share('cleancache', [$this, 'getControllerCleanCacheService'], \true);
        $container->alias(Configurations::class, 'configurations')->share('configurations', [$this, 'getControllerConfigurationsService'], \true);
        $container->alias(DeleteBackups::class, 'deletebackups')->share('deletebackups', [$this, 'getControllerDeleteBackupsService'], \true);
        $container->alias(Help::class, 'help')->share('help', [$this, 'getControllerHelpService'], \true);
        $container->alias(KeyCache::class, 'keycache')->share('keycache', [$this, 'getControllerKeyCacheService'], \true);
        $container->alias(Main::class, 'main')->share('main', [$this, 'getControllerMainService'], \true);
        $container->alias(OptimizeImage::class, 'optimizeimage')->share('optimizeimage', [$this, 'getControllerOptimizeImageService'], \true);
        $container->alias(OptimizeImages::class, 'optimizeimages')->share('optimizeimages', [$this, 'getControllerOptimizeImagesService'], \true);
        $container->alias(OrderPlugins::class, 'orderplugins')->share('orderplugins', [$this, 'getControllerOrderPluginsService'], \true);
        $container->alias(RestoreImages::class, 'restoreimages')->share('restoreimages', [$this, 'getControllerRestoreImagesService'], \true);
        $container->alias(ToggleSetting::class, 'togglesetting')->share('togglesetting', [$this, 'getControllerToggleSettingService'], \true);
        $container->alias(PageCache::class, 'pagecache')->share('pagecache', [$this, 'getControllerPageCacheService'], \true);
        // Models
        $container->share(Configure::class, [$this, 'getModelConfigureService'], \true);
        $container->share(Cache::class, [$this, 'getModelMainService'], \true);
        $container->share(PageCacheModel::class, [$this, 'getModelPageCacheModelService'], \true);
        // Views
        $container->share(HtmlView::class, [$this, 'getViewHtmlService'], \true);
        $container->share(MainHtml::class, [$this, 'getViewMainHtmlService'], \true);
        $container->share(ConfigurationsHtml::class, [$this, 'getViewConfigurationsHtmlService'], \true);
        $container->share(PageCacheHtml::class, [$this, 'getViewPageCacheHtmlService'], \true);
    }

    public function getInputService(Container $container): Input
    {
        return new Input($_REQUEST);
    }

    public function getWordpressNoticeLoggerService(Container $container): WordpressNoticeLogger
    {
        return new WordpressNoticeLogger();
    }

    public function getControllerResolverService(Container $container): ControllerResolver
    {
        return new ControllerResolver($container, $container->get(Input::class));
    }

    public function getTemplateRendererService(Container $container): RendererInterface
    {
        $renderer = new BladeRenderer($container->get(Factory::class));
        $renderer->addFolder(Paths::templatePath());

        return $renderer;
    }

    public function getControllerApplyAutoSettingService(Container $container): ApplyAutoSetting
    {
        $controller = new ApplyAutoSetting($container->get(Configure::class), $container->get(Input::class));
        $controller->setLogger($container->get(WordpressNoticeLogger::class));

        return $controller;
    }

    public function getControllerBrowserCachingService(Container $container): BrowserCaching
    {
        $controller = new BrowserCaching($container->get(Input::class));
        $controller->setLogger($container->get(WordpressNoticeLogger::class));

        return $controller;
    }

    public function getControllerCleanCacheService(Container $container): CleanCache
    {
        $controller = new CleanCache($container->get(Cache::class), $container->get(Input::class));
        $controller->setLogger($container->get(WordpressNoticeLogger::class));

        return $controller;
    }

    public function getControllerConfigurationsService(Container $container): Configurations
    {
        return new Configurations($container->get(ConfigurationsHtml::class), $container->get(Input::class));
    }

    public function getControllerDeleteBackupsService(Container $container): DeleteBackups
    {
        $controller = new DeleteBackups($container->get(Input::class));
        $controller->setLogger($container->get(WordpressNoticeLogger::class));

        return $controller;
    }

    public function getControllerHelpService(Container $container): Help
    {
        return new Help($container->get(HtmlView::class), $container->get(Input::class));
    }

    public function getControllerKeyCacheService(Container $container): KeyCache
    {
        $controller = new KeyCache($container->get(Input::class));
        $controller->setLogger($container->get(WordpressNoticeLogger::class));

        return $controller;
    }

    public function getControllerMainService(Container $container): Main
    {
        return new Main($container->get(MainHtml::class), $container->get(Cache::class), $container->get(Icons::class), $container->get(Input::class));
    }

    public function getControllerOptimizeImageService(Container $container): OptimizeImage
    {
        $controller = new OptimizeImage($container->get(Input::class));
        $controller->setLogger($container->get(WordpressNoticeLogger::class));

        return $controller;
    }

    public function getControllerOptimizeImagesService(Container $container): OptimizeImages
    {
        return new OptimizeImages($container->get(HtmlView::class), $container->get(Icons::class), $container->get(Input::class));
    }

    public function getControllerOrderPluginsService(Container $container): OrderPlugins
    {
        $controller = new OrderPlugins($container->get(Loader::class, Input::class));
        $controller->setLogger($container->get(WordpressNoticeLogger::class));

        return $controller;
    }

    public function getControllerRestoreImagesService(Container $container): RestoreImages
    {
        $controller = new RestoreImages($container->get(Input::class));
        $controller->setLogger($container->get(WordpressNoticeLogger::class));

        return $controller;
    }

    public function getControllerToggleSettingService(Container $container): ToggleSetting
    {
        $controller = new ToggleSetting($container->get(Configure::class), $container->get(Input::class));
        $controller->setLogger($container->get(WordpressNoticeLogger::class));

        return $controller;
    }

    public function getControllerPageCacheService(Container $container): PageCache
    {
        $controller = new PageCache($container->get(PageCacheHtml::class), $container->get(PageCacheModel::class), $container->get(Input::class));
        $controller->setLogger($container->get(WordpressNoticeLogger::class));

        return $controller;
    }

    public function getModelConfigureService(Container $container): Configure
    {
        $model = new Configure();
        $model->setState($container->get('params'));
        $model->setContainer($container);

        return $model;
    }

    public function getModelMainService(Container $container): Cache
    {
        return (new Cache($container->get(StorageInterface::class), $container->get(CorePageCache::class), $container->get(TaggableInterface::class)))->setContainer($container);
    }

    public function getModelPageCacheModelService(Container $container): PageCacheModel
    {
        return new PageCacheModel($container->get(Input::class), $container->get(CorePageCache::class), $container);
    }

    public function getViewHtmlService(Container $container): HtmlView
    {
        $view = new HtmlView($container->get(RendererInterface::class));
        $view->setLayout($container->get(Input::class)->get('tab', 'main'));

        return $view;
    }

    public function getViewMainHtmlService(Container $container): MainHtml
    {
        return (new MainHtml($container->get(RendererInterface::class)))->setLayout('main');
    }

    public function getViewConfigurationsHtmlService(Container $container): ConfigurationsHtml
    {
        return (new ConfigurationsHtml($container->get(RendererInterface::class)))->setLayout('configurations');
    }

    public function getViewPageCacheHtmlService(Container $container): PageCacheHtml
    {
        return new PageCacheHtml($container->get(RendererInterface::class));
    }
}
